#!/usr/bin/perl

#Author: Jamie Duke
#This is the implementation of GOR to predict the secondary structure of 
#a protein

use strict vars;

#check the command line arguments
if ( @ARGV !=2 ) {
	die "Usage: gor.pl <database file> <protein string>\n";
}

#open the database file for reading
open DB, "<$ARGV[0]" or die "Cannot open the file $ARGV[0]: $!\n";

#create global variables for the hashes of secondary structure info
our %C; our %H; our %E; 

#read in the database file -- split each line into the structure type
#the amino acid identifier and then the information number for each
#position in the window then put it in the correct hash.
while( my $line = <DB> ) {
	(my $ss, my $aa, my @num) = split(/\t/, $line);
	${$ss}{$aa} = [@num];	
}

#variable to hold the secondary structure prediction
my $pss; 

#split the sequence into any array
my @seq = split(/(?=\w)/, $ARGV[1]);

#loop throught the sequence one residue at a time
for( my $i = 0; $i < @seq; $i++ ) {

	#variables for each type of structure
	my $h = 0; my $c = 0; my $e = 0;  

	#loop through the window
	for( my $j = -8; $j < 17; $j++ ) {

		#only look at the residues with an index > 0 or
		#less than the length of the sequence
		if( ($i+$j > 0) && ($i+$j < @seq) ) {

			#for each secondary structure type add up 
			#the information values
			$h += $H{$seq[$i+$j]}[$j+8];
			$c += $C{$seq[$i+$j]}[$j+8];
			$e += $E{$seq[$i+$j]}[$j+8];
		}
	}

	#find the maximum of the three secondary structures and define
	#that as the secondary structure prediction for the residue 
	#if there is a tie -- resolve it by marking it as an X
	if($h > $c && $h > $e) {
		$pss .= "H";
	} elsif( $c > $h && $c > $e) {
		$pss .= "C";
	} elsif( $e > $h && $e > $c) {
		$pss .= "E";
	} else {
		$pss .="X";
	}
}

#print out the sequence and the prediction
print "Your secondary structure prediction:\n";
print join('', @seq) ."\n$pss\n";
