#!/usr/bin/perl

#This script is to go through all the pdb files with the predetermined chain and create 
#secondary structure information

if( @ARGV != 3 ) {
	die "Usage: <identifier file> <DSSP file> <output file>\n";
}

#Arguments
#<file containing all pdb codes followed by a '.' then a chain identifier

#open the files for either reading or writing
open REF, "<$ARGV[0]" or die "Cannot open the file: $!\n";
open DSSP, "<$ARGV[1]" or die "Cannot open the file: $!\n";
open OUT, ">$ARGV[2]" or die "Cannot open the file: $!\n";

#create a hash to hold all of the pdb codes and the chain identifiers
%ids;

#read through the file
while( $i = <REF> ) {

	#Break up the line into the pdb code($1) and the chain identifier ($2)
        $i =~ /(\w{4})\.(\w)/;

	#if the chain identifier is X then change it to an underscore to 
	#match the DSSP chain identifier otherwise just store the chain
	if( $2 eq  "X") {
		$input = "_";
	} else {
		$input = $2;
	}
 
	#some pdb codes have multiple chains of interest, so if a key already
	#exists for the key you want to make, just add the chain onto 
	#the pre-existing value otherwise just add the entry
        if(exists($ids{$1})) {
		$ids{$1} = $ids{$1} .",".$input;
	}else {
		$ids{$1} = $input;
	}
}

#close the file that we were just reading
close REF;

#scalar to hold the total number of residues in the "database"
$total_res = 0;

#create a hash to hold the counts of each type of amino acid in all sequences
%res = ( A => 0, C => 0, D => 0, E => 0, F => 0, G => 0, H => 0, I => 0, K => 0,
	L => 0, M => 0, N => 0, P => 0, Q => 0, R => 0, S => 0, T => 0, V => 0,
	W => 0, Y => 0,);

#define the different arrays for the three types of secondary structure
foreach $r ( ("H", "C", "E")) {
%{$r} = (
	A => [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0],
	C => [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0],
	D => [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0],
	E => [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0],
	F => [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0],
	G => [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0],
	H => [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0],
	I => [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0],
	K => [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0],
	L => [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0],
	M => [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0],
	N => [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0],
	P => [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0],
	Q => [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0],
	R => [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0],
	S => [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0],
	T => [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0],
	V => [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0],
	W => [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0],
	Y => [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0],
);
}

#make arrays for each type of secondary structure to hold the number of 
#times that the position is seen for the type of structure
@H = (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0);
@C = (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0);
@E = (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0);

#make scalar variables to hold the count of each type of secondary structure
$H = 0;
$C = 0;
$E = 0;


#loop through the DSSP file
while($line = <DSSP>) {

	#if the line contains the identifier get it and store the 
	#chains that were found in the first input file
	if( $line =~ /^ID\s*: (\w{4})/){
		$pid = $1;
		$chz = $ids{$1};
	}
	
	#if the line is equal to the chain being described store it
	if( $line =~ /^Chain\s*: (\w)/) {
		$cz = $1;
	}

	#if we are looking at a chain that was picked out from the first
	#input file
	if( $chz =~ /$cz/ ) {
		
		#if the line is sequence then store the sequence
		if( $line =~ /^ Sequence\s*: (\w+)/) {
			$seq = $1;

			if( $seq =~ /X/ ) {
				print "pid = $pid chain = $cz\n";
			}
		}

		#if the line contains the DSSP characters
		if( $line =~ /^ DSSP\s*: (\w+)/) { 
		
			#store the secondary structure information but only 
			#after simplifying it to H, E, and C.	
			$str = &simplify($1);
			
			#then run the subroutine to sort out all of the 
			#secondary structure information
			&getss( $seq, $str);
		}
	}
}

#close the secondary structure file
close DSSP;

#go through each hash of secondary structure and calculate the 
#information function and print it out to the output file
foreach $r ( ("C", "E", "H") ) {

	#loop through each of the residues for the given secondary structure
	foreach $k (keys %{$r}){

	#start the print of the output file with the secondary structure 
	#feature and the name of the residue
	print OUT "$r\t$k";

	#loop through the window
	for( $i = 0; $i <17; $i++) {

		#make sure that these two values are positive otherwise
		#there could be the possibility of dividing by 0
		#this is very unlikely with a large database
		if( ${$r}[$i] > 0 && $res{$k} > 0) {		

		     #calculate the information function:
		     # (# of given AA seen in window position i ) / ( # of 
		     # times secondary structure applies to position i)
		     # multiplied by ( total number of residues seen / 
		     # # of times given AA was seen in database)
		     $t =(${$r}{$k}[$i] / ${$r}[$i] ) * ($total_res / $res{$k});

			#make sure that $t is not 0 as you can't take the 
			#log of 0
			if( $t > 0) {
				print OUT "\t" . &log10( $t );
			} else {
				#this number should never be seen, but it 
				#exists just in case 
				print OUT "\t" . &log10(0.0000001);
			}

		# if you are going to divide by 0, then take the log of a very
		# small number to make the information function very unlikely at
		# that position -- once again shouldn't be seen if the database 
		# is large but it exists just in case
		} else { 
			print OUT "\t" . &log10(0.000000001);
		}
	}  print OUT "\n"; 
	} 
}

#close the output file
close OUT;

####################### SUBROUTINES ################################3

#This subroutine is the meat of the program
#What it does -- given two strings one of sequence and the other of secondary
#structure, it goes through and creates the counts needed for the information
#calculation
#
#Arguments: 2 arguments
# Arg1 = sequence string
# Arg2 = structure string
#Returns: Nothing
 
sub getss {
	
	#create arrays for both arguments -- easier to manipulate than strings	
	@seq = split(/(?=\w)/, $_[0]);
	@str = split(/(?=\w)/, $_[1]);
	
	#record the total length of the sequence
	$length = @str;

	#double loop to go through each position in the sequence and each 
	#position in the 17 residue window
	for( $i = 0; $i<@str; $i++) {

		#count the number of each amino acid in the sequence
		$res{$seq[$i]}++;

		#count the number of each type of secondary structure
		${$str[$i]}++;

	for( $j = -8; $j<=8;  $j++) {

		#check to make sure that the index is not negative 
		#or not off the end of the array
		if( ($i + $j >= 0) && ($i + $j < $length) ) {

			#Increase the count of the number of times the
			#secondary structure is seen for the given position
			#in the 17 residue window
			${$str[$i]}[$j+8]++;

			#Increase the array at position of the window +8 
			#(so the index is base 0) of the secondary structure
			#hash either %H, %C or %E. 

			${$str[$i]}{$seq[$i+$j]}[$j+8] = ${$str[$i]}{$seq[$i+$j]}[$j+8] +1;
		}
	}
	}

	#increase the total number of residues that are in the database
	$total_res += $length;
}

#This subroutine replaces some of the less common secondary structures from 
#DSSP and replaces them with the symbol for coils 'C'
#for reference: B = Beta Bridge, G = 3/10 helix, I = pi helix, 
#	        T = hydrogen bonded turn, S = bend
#Argument: 1 Argument
# Arg1: string of secondary structure characters
#Returns: string of modified secondary structure characters

sub simplify {
	$str = $_[0];
	$str =~ s/B/C/g;
	$str =~ s/G/C/g;
	$str =~ s/I/C/g;
	$str =~ s/T/C/g;
	$str =~ s/S/C/g;

	return $str;
}

#Goes through a given array and sums the numbers in the array
#Arguments: 1 Argument
# Arg1: reference to the array
#Returns the sum of its indices
sub countup {
	$ref = shift;
	$sum = 0;
	foreach $r (@{$ref}) {
		$sum += $r;
	}
	return sum; 
}


#this subroutine calculates the log of a number base 10 instead of base e
#Arguments: 1 Argument
# Arg1: scalar with number to take the log of
#Retursn: scalar with the log on the number
sub log10 {
	my $n = shift;
	return log($n)/log(10);
}
