#!/usr/bin/perl

#Author: Jamie Duke
#For Genomics & Bioinformatics Fall 2006

#This is to take the large DSSP PDBFIND2.txt file and extract the files that
#are needed for the database for GOR

if( @ARGV != 3 ) {
	die "USAGE: makesmall.pl <pdb_identifier file> <DSSP database file> <output file>\n";
}

#open the files from the command line argument
open REF, "<$ARGV[0]" or die "Cannot open $ARGV[0]: $!\n";
open DSSP, "<$ARGV[1]" or die "Cannot open $ARGV[1]: $!\n";
open OUT, ">$ARGV[2]" or die "Cannot open $ARGV[2]: $!\n";

#create a hash with the pdb code and a chain identifier
#number of keys will be less than the # of chains due to the fact
#that some ids have multiple chains
%ids;
while( $i = <REF> ) {
	$i =~ /(\w{4})\.(\w)/;
	$ids{$1} = $2;
}	
close REF;

#loop through the DSSP file
while( $i = <DSSP>) {

	#if the line contains the PDB identifier
	if( $i =~/^ID\s*: (\w{4})/){

		#if the identifier is one that we are interested in.
		if(exists($ids{$1})){

			#if we are interested print the current line and 
			#continue adding lines to the output file while 
			#you don't see a line that has '//' as that is 
			#the end of the current identifier
			print OUT $i;
			while( $i = <DSSP>) {
				print OUT $i;
				if( $i =~ /\/\//){ last(); }
			}
		}
	}
}

#close input file and output file
close DSSP;
close OUT;

